//
//
// ASASClient skeleton driver - TroSoft AB (c) 2013-2014
//
// Please replace the CompanyName constant with your own company name
//
// TroSoft AB have no responsibility whatsover if there are bugs in any other
// inconsistency in the code, it is only an example of how to implement the
// Alarmus Alarm Source Server / Client solution using the simple JSon protocol
//
// The program may be installed as service using ASASClient /i and uninstalled
// using ASASClient /u. The installation should be performed using a command prompt
// started as administrator
//
// Date     / Sign / Vers    / Comment
// ---------------------------------------------------------------------------------------------------
// 14.04.01 / TR / 1.0.0.00  / Main - runs
// 14.08.28 / TR / 1.0.0.01  / SendPacket returned false even if it succeeded
// ---------------------------------------------------------------------------------------------------
//

using System;
using System.Threading;
using System.IO;
using System.Text;
using Microsoft.Win32;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Collections.Generic;
using System.Web.Script.Serialization;

namespace nsASASClient
{

  public class ASASClient
	{

    static public string CompanyName = "TroSoft"; // Should be replaced with the company name

    const int CleanupLoopInterval = 100;

    static public string ServiceName = "";
    static public string ServiceDescription = "";

    static public System.Diagnostics.EventLog AppEventLog;
    static public SysLogAndDebug SysLog = new SysLogAndDebug();

    static public bool ThreadsShouldRun = true;
    static public bool RunHasStopped = false;

    static public cTcpSocketClient ASASServerConnection;

    static public StreamWriter swDebugFile = null;

    static public string sPrimaryServerAddress;
    static public string sSecondaryServerAddress;

    static public int iAckEchoTimeout;
    static public int iEchoTestInterval;
    static public int iConnectInterval;

    static public int iNextEventType = 0;
    static public string sAlarmTag = "";

    static public JavaScriptSerializer JSonSerializer = new JavaScriptSerializer();

 		static public void Run()
		{

			int iDemoAlarmTimer = 0;

			Random randObj = new Random();

      iEchoTestInterval = PrivateProfile.GetIniFileInt("General", "EchoTestInterval", 30);
      iAckEchoTimeout = PrivateProfile.GetIniFileInt("General", "AckEchoTimeout", 15);

      sPrimaryServerAddress = PrivateProfile.GetIniFileString("General", "PrimaryServerAddress", "");
      sSecondaryServerAddress = PrivateProfile.GetIniFileString("General", "SecondaryServerAddress", "");

      iConnectInterval = PrivateProfile.GetIniFileInt("General", "ConnectInterval", 15);

      // Convert to msecs
      iEchoTestInterval = (iEchoTestInterval > 0) ? iEchoTestInterval * 1000 : 0;
      iAckEchoTimeout = (iAckEchoTimeout > 0) ? iAckEchoTimeout * 1000 : 0;
      iConnectInterval = (iConnectInterval > 0) ? iConnectInterval * 1000 : 0;

			ASASServerConnection = new cTcpSocketClient();
      ASASServerConnection.Startup(sPrimaryServerAddress, sSecondaryServerAddress, iConnectInterval, iEchoTestInterval, iAckEchoTimeout);

			SysLog.SysLog("ASAS where started");

			while (ThreadsShouldRun)
			{
        System.Threading.Thread.Sleep(CleanupLoopInterval);

				//
				// Get the alarms from somewhere (a file, API or whatever) and send them away
				//
				// We send some random alarms just to demonstrate how it works
				//

				iDemoAlarmTimer += CleanupLoopInterval;

				if (iDemoAlarmTimer >= 30000)
				{
					//
					// Fill in necessary fields
					//
					ASAS_JSon_Simple_Messages.AlarmEvent alarmEvent = new ASAS_JSon_Simple_Messages.AlarmEvent();
          // Will be seet when class is created anyway but to show an example
					alarmEvent.TimeStamp = ASAS_JSon_Simple_Messages.ISO8601.CreateISO8601UTCTimeStamp(DateTime.Now);
          switch (iNextEventType)
          {
            case 0:
              sAlarmTag = "TESTTAG_" + randObj.Next(1000).ToString("D3");
              alarmEvent.EventType = "Activate";
              iNextEventType++;
              break;
            case 1:
              alarmEvent.EventType = "Inactivate";
              iNextEventType++;
              break;
            case 2:
              alarmEvent.EventType = "Acknowledge";
              iNextEventType = 0;
              break;
          }
					alarmEvent.Tag = sAlarmTag;
					alarmEvent.Name = "Pump stop or something";
					alarmEvent.Description = "Alarm description - some error occured somewhere";
					alarmEvent.Priority = 2;
          if (ASASServerConnection.SendPacket(JSonSerializer.Serialize(alarmEvent)) == false)
          {
            // Actually if we cannot send this packet we should remember the event and store it somewhere - at least if we are not capable of
            // providing all alarm objects and their status when Alarmus Server requests them (NotSupported)
            // When we are connected againg we send all buffered events
          }
					iDemoAlarmTimer = 0;
				}
				ASASServerConnection.CyclicCleanup(CleanupLoopInterval);
				SysLog.CyclicCleanup(CleanupLoopInterval);
			}

      SysLog.SysLog("Closing sockets and threads");

      ASASServerConnection.Shutdown();

      if (swDebugFile != null)
      {
        swDebugFile.Close();
      }

			SysLog.SysLog("ASAS have been stopped");

      // Just in case
      Thread.Sleep(100);

      RunHasStopped = true;

		}

    static public void ReceivedPacketFromASASServer(string sJSon)
    {

			ASAS_JSon_Simple_Messages.Header header;

			try
			{
				header = JSonSerializer.Deserialize<ASAS_JSon_Simple_Messages.Header>(sJSon);
        if (header.MsgType == null)
        {
          SysLog.SysLog("JSon header was bad: {0}", sJSon);
        }
        switch (header.MsgType.ToLower())
				{

				  case "alarmcommand":
						// Got some command - deserialise and do something about it if we have some
						// support for it (ack an alarm or disable or whatever)
            ASAS_JSon_Simple_Messages.AlarmCommand alarmCommand = JSonSerializer.Deserialize<ASAS_JSon_Simple_Messages.AlarmCommand>(sJSon);
            switch (alarmCommand.Command.ToLower())
            {
              case "acknowledge":
                // Do something
                break;
            }
            break;

          case "servercommandandresponse":
            ASAS_JSon_Simple_Messages.ServerCommandAndResponse serverCommandAndResponse = JSonSerializer.Deserialize<ASAS_JSon_Simple_Messages.ServerCommandAndResponse>(sJSon);
            switch (serverCommandAndResponse.Command.ToLower())
            {
              case "requestallalarmobjects":
                // Here we should send PrepareForFirstAlarmObject then all alarm objects then LastAlarmObjectHasBeenSent (if supported)
                // If not we just send NotSupported and then all alarm events we have buffered when the connection was down
                // In this sample skeleton client we just dont send anything
                ASASServerConnection.SendPacket(ASASClient.JSonSerializer.Serialize(new ASAS_JSon_Simple_Messages.ServerCommandAndResponse(null, "NotSupported")));
                break;
              default:
                ASASServerConnection.SendPacket(ASASClient.JSonSerializer.Serialize(new ASAS_JSon_Simple_Messages.ServerCommandAndResponse(null, "NotSupported")));
                break;
            }
          break;
				}
			}
			catch (Exception e)
			{
        SysLog.SysLog("JSon header was bad: {0} ({1})", sJSon, e.Message);
			}
		}

    static void Main(string[] saArgs)
    {

      if (File.Exists(PrivateProfile.IniFileFullname()) == false)
      {
        Console.WriteLine("The INI file " + PrivateProfile.IniFileFullname() + " could not be found!");
        return;
      }
      
      ASASClient.ServiceName = PrivateProfile.GetIniFileString("General", "ServiceName", "ASASClient");
      ASASClient.ServiceDescription = PrivateProfile.GetIniFileString("General", "ServiceDescription", ASASClient.CompanyName + " Alarmus Standard Alarm Source Client (ASASClient)");

      foreach (string sArgument in saArgs)
      {
        // Install service ?
        if (sArgument.ToUpper() == "/I" || sArgument.ToUpper() == "-I")
        {
          try
          {
            ServiceHelpStuff.InstallService(ASASClient.ServiceName, ASASClient.ServiceDescription, "", "");
            string sInfo = ASASClient.ServiceName + " was successfully installed as service. Please start it from Service Control Manager first time.";
            Console.WriteLine(sInfo);
          }
          catch (Exception e)
          {
            Console.WriteLine("Failed to install service: " + e.Message);
          }

          return;
        }
        // Uninstall service ?
        if (sArgument.ToUpper() == "/U" || sArgument.ToUpper() == "-U")
        {
          try
          {
            ServiceHelpStuff.UnInstallService(ASASClient.ServiceName);
            string sInfo = ASASClient.ServiceName + " was successfully uninstalled as service";
            Console.WriteLine(sInfo);
          }
          catch (Exception e)
          {
            Console.WriteLine("Failed to uninstall service: " + e.Message);
          }
          return;
        }
      }

      try
      {
        Directory.CreateDirectory(PrivateProfile.SysLogFilesPath());
      }
      catch { }

      try
      {
        Directory.CreateDirectory(PrivateProfile.DebugFilesPath());
      }
      catch { }

      if (ServiceHelpStuff.IsServiceInstalled(ASASClient.ServiceName))
      {
        SysLog.SysLog(ASASClient.ServiceName + " is starting as service");
        SysLog.SysLog("Using INI-file '" + PrivateProfile.IniFileFullname() + "'");
        System.ServiceProcess.ServiceBase[] ServicesToRun;
        ServicesToRun = new System.ServiceProcess.ServiceBase[] { new ASASClientService() };
        System.ServiceProcess.ServiceBase.Run(ServicesToRun);
      }
      else
      {
        ASASClientService service = new ASASClientService();
        SysLog.SysLog(ASASClient.ServiceName + " is starting as a normal application in console mode");
        SysLog.SysLog("Using INI-file '" + PrivateProfile.IniFileFullname() + "'");
        service.Start();
        Console.WriteLine("Press ESC to stop application");
        while (Console.ReadKey().Key != ConsoleKey.Escape)
        {
        }
        SysLog.SysLog(ASASClient.ServiceName + " is stopping (is running as a normal application in console mode)");
        service.Stop();
        SysLog.SysLog(ASASClient.ServiceName + " was stopped (has run as a normal application in console mode)");
        SysLog.SysLog("");
      }
    }
	}
}
