﻿using System;
using System.Globalization;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ASAS_JSon_Simple_Messages
{

  // Just used for making decoding simple
  public class Header
  {
    public string MsgType; // AlarmEvent / AlarmCommand / AlarmObject / EchoTest / EchoAck / ServerCommandAndResponse
  }

  public class EchoTest
  {
    public string MsgType = "EchoTest";
  }

  public class EchoAck
  {
    public string MsgType = "EchoAck";
  }

  public class AlarmEvent
  {

    //
    // This class is sent from the ALASClient to the ALASServer,
    // usually as a result of some change in the alarm status,
    // ex the alarm goes from normal -> alarm (would result in an Activate message)
    //
    public string MsgType = "AlarmEvent";    // AlarmEvent
    public string EventType;  // Acknowledge / Activate / Inactivate / Delete / Disable / Enable
    public string TimeStamp;
    public string Tag;
    public string Name;
    public string Description;
    public string UserName;
    public int Priority;
    public AlarmEvent()
    {
      TimeStamp = ISO8601.CreateISO8601UTCTimeStamp(DateTime.Now);
    }
  }

  //
  // This class is sent from the ALASServer to the ALASClient (and then possibly to the SCADA system,
  // usually as a result of some user interaction in the user interface (AlarmClient) ex the user acknowledges an alarm
  //
  public class AlarmCommand
  {
    public string MsgType = "AlarmCommand";  // AlarmCommand
    public string Command;  // Acknowledge / Activate / Inactivate / Delete / Disable / Enable
    public string TimeStamp;
    public string Tag;
    public string UserName;
  }

  //
  // RequestAllAlarmObjects is sent when the ASASServer is connected to the Alarmus Server and
  // the Alarm Source is enabled - as a result all AlarmObjects should be sent to ASASServer.
  // Begin with a PrepareForFirstAlarmObject before first AlarmObject and end with a LastAlarmObjectHasBeenSent
  // when last AlarmObject has been sent.
  // If the client does not support this functionality, the client should send only a NotSupported message and then
  // all buffered alarm events (as events)

  //

  public class ServerCommandAndResponse
  {
    public string MsgType = "ServerCommandAndResponse";
    public string Command; // RequestAllAlarmObjects / PrepareForFirstAlarmObject / LastAlarmObjectHasBeenSent
    public string Response; // NotSupported
    // Allow also null
    public ServerCommandAndResponse()
    {
    }
    public ServerCommandAndResponse (string sCommand, string sResponse)
    {
      Command = sCommand;
      Response = sResponse;
    }
  }

  //
  // This class is sent when the ALASClient is connected to the ALASServer and the ALASServer request all
  // alarm objects (to get some initial status)
  //

  public class AlarmObject
  {
    public string MsgType = "AlarmObject";    // AlarmObject
    public string Status;     // Normal / ActiveUnacked / ActiveAcked / InactiveUnacked / Disabled / Removed
    public string ActiveTimeStamp;
    public string AckedTimeStamp;
    public string InactiveTimeStamp;
    public string DisabledTimeStamp;
    public string DisabledExpireTime;
    public string Tag;
    public string Name;
    public string Description;
    public string LastAckedByUserName;
    public string LastDisabledByUserName;
    public string LastEnabledByUserName;
    public int Priority;
  }


  public static class ISO8601
  {

    public static string CreateISO8601UTCTimeStamp(DateTime dtTimeStamp)
    {
      string sTimeStamp = String.Format("{0:yyyy-MM-dd}T{0:HH:mm:ss.fff}Z", dtTimeStamp.ToUniversalTime());
      return sTimeStamp;
    }

    public static DateTime UnpackISO8601UTCTimeStamp(ref string sDateTime)
    {

      DateTime dtReturnDateTime;

      try
      {
        dtReturnDateTime = DateTime.ParseExact(sDateTime.ToUpper(), @"yyyy-MM-dd\THH:mm:ss.fff\Z", CultureInfo.InvariantCulture, DateTimeStyles.AssumeUniversal);
      }
      catch
      {
        try
        {
          dtReturnDateTime = DateTime.ParseExact(sDateTime.ToUpper(), @"yyyy-MM-dd\THH:mm:ss\Z", CultureInfo.InvariantCulture, DateTimeStyles.AssumeUniversal);
        }
        catch
        {
          dtReturnDateTime = DateTime.Now;
        }
      }

      return dtReturnDateTime;
    }

  }


}

