﻿using System;
using System.Reflection;
using System.Collections;
using System.Threading;
using System.IO;
using System.Text;
using Microsoft.Win32;
using System.ServiceProcess;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace nsASASClient
{

  public class ASASClientService : System.ServiceProcess.ServiceBase
  {
    /// <summary> 
    /// Required designer variable.
    /// </summary>
    private System.ComponentModel.Container components = null;
    private Thread RunThread;

    public ASASClientService()
    {

      this.AutoLog = false;
      this.CanStop = true;
      this.CanShutdown = true;

      // This call is required by the Windows.Forms Component Designer.
      InitializeComponent();

    }

    private void InitializeComponent()
    {
      components = new System.ComponentModel.Container();
      this.ServiceName = ASASClient.ServiceName;
    }

    protected override void Dispose(bool disposing)
    {
      if (disposing)
      {
        if (components != null)
        {
          components.Dispose();
        }
      }
      base.Dispose(disposing);
    }

    protected override void OnStart(string[] args)
    {

      ASASClient.AppEventLog = new System.Diagnostics.EventLog("Application", ".", this.ServiceName);

      Start();

      ASASClient.AppEventLog.WriteEntry(ASASClient.ServiceName + " was started", System.Diagnostics.EventLogEntryType.Information);
      ASASClient.SysLog.SysLog(ASASClient.ServiceName + " was started as a Service");

    }

    // Kick off a new thread to do the actual stuff
    public void Start()
    {
      RunThread = new Thread(new ThreadStart(ASASClient.Run));
      RunThread.Start();
    }

    /// <summary>
    /// Stop this service.
    /// </summary>
    protected override void OnStop()
    {

      ASASClient.SysLog.SysLog(ASASClient.ServiceName + " is stopping (running as Service)");
      ASASClient.AppEventLog.WriteEntry(ASASClient.ServiceName + " is stopping", System.Diagnostics.EventLogEntryType.Information);

      Stop();

      ASASClient.AppEventLog.WriteEntry(ASASClient.ServiceName + " was stopped", System.Diagnostics.EventLogEntryType.Information);
      ASASClient.SysLog.SysLog(ASASClient.ServiceName + " was stopped (was run as Service)");
      ASASClient.SysLog.SysLog("");

    }

    protected override void OnShutdown()
    {

      ASASClient.SysLog.SysLog(ASASClient.ServiceName + " is stopping  due to system shutdown (running as Service)");

      // Try section though event viewer may be down when we close
      try
      {
        ASASClient.AppEventLog.WriteEntry(ASASClient.ServiceName + " is stopping due to system shutdown", System.Diagnostics.EventLogEntryType.Information);
      }
      catch { }

      Stop();

      try
      {
        ASASClient.AppEventLog.WriteEntry(ASASClient.ServiceName + " was stopped due to system shutdown", System.Diagnostics.EventLogEntryType.Information);
      }
      catch { }

      ASASClient.SysLog.SysLog(ASASClient.ServiceName + " was stopped due to system shutdown (was run as Service)");
      ASASClient.SysLog.SysLog("");

    }

    //
    // Stop the Run() loop
    //
    public new void Stop()
    {
      ASASClient.ThreadsShouldRun = false;

      while (ASASClient.RunHasStopped == false)
      {
        System.Threading.Thread.Sleep(20);
      }
    }

  }

  class ServiceHelpStuff
  {

    #region DLL Library Imports

    /// <summary>

    /// Import functions from the service control manager API.

    /// </summary>

    [StructLayout(LayoutKind.Sequential)]

    public struct SERVICE_DESCRIPTION
    {

      public string lpDescription;

    }

    [DllImport("advapi32.dll", SetLastError = true, EntryPoint = "ChangeServiceConfig2")]
    public static extern int ChangeServiceDescription(IntPtr hService, int dwInfoLevel, [MarshalAs(UnmanagedType.Struct)] ref SERVICE_DESCRIPTION lpInfo);

    [DllImport("advapi32.dll", SetLastError = true)]
    public static extern IntPtr OpenSCManager(string lpMachineName, string lpSCDB, int scParameter);

    [DllImport("Advapi32.dll", SetLastError = true)]
    public static extern IntPtr CreateService(IntPtr SC_HANDLE, string lpSvcName, string lpDisplayName, int dwDesiredAccess, int dwServiceType, int dwStartType, int dwErrorControl, string lpPathName, string lpLoadOrderGroup, int lpdwTagId, string lpDependencies, string lpServiceStartName, string lpPassword);

    [DllImport("advapi32.dll", SetLastError = true)]
    public static extern void CloseServiceHandle(IntPtr SCHANDLE);

    [DllImport("advapi32.dll", SetLastError = true)]
    public static extern int StartService(IntPtr SVHANDLE, int dwNumServiceArgs, string lpServiceArgVectors);

    [DllImport("advapi32.dll", SetLastError = true)]
    public static extern IntPtr OpenService(IntPtr SCHANDLE, string lpSvcName, int dwNumServiceArgs);

    [DllImport("advapi32.dll", SetLastError = true)]
    public static extern int DeleteService(IntPtr SVHANDLE);

    [DllImport("kernel32")]
    public static extern int GetModuleFileName(int iHandle, StringBuilder lpFilePath, int iFilePathLength);

    #endregion DLL Library Imports


    #region Constant declarations required by the service control API.

    /// <summary>

    /// constants required by the service control library.

    /// </summary> 

    private const int SERVICE_WIN32_OWN_PROCESS = 0x00000010;
    private const int SERVICE_DEMAND_START = 0x00000003;
    private const int SERVICE_ERROR_NORMAL = 0x00000001;


    private const int STANDARD_RIGHTS_REQUIRED = 0xF0000;

    private const int SERVICE_QUERY_CONFIG = 0x0001;
    private const int SERVICE_CHANGE_CONFIG = 0x0002;
    private const int SERVICE_QUERY_STATUS = 0x0004;
    private const int SERVICE_ENUMERATE_DEPENDENTS = 0x0008;
    private const int SERVICE_START = 0x0010;
    private const int SERVICE_STOP = 0x0020;
    private const int SERVICE_PAUSE_CONTINUE = 0x0040;
    private const int SERVICE_INTERROGATE = 0x0080;
    private const int SERVICE_USER_DEFINED_CONTROL = 0x0100;
    private const int SERVICE_DELETE = 0x10000;

    private const int SC_MANAGER_CONNECT = 0x0001;
    private const int SC_MANAGER_CREATE_SERVICE = 0x0002;
    private const int SC_MANAGER_ENUMERATE_SERVICE = 0x0004;
    private const int SC_MANAGER_LOCK = 0x0008;
    private const int SC_MANAGER_QUERY_LOCK_STATUS = 0x0010;
    private const int SC_MANAGER_MODIFY_BOOT_CONFIG = 0x0020;

    private const int SERVICE_CONFIG_DESCRIPTION = 0x0001;


    private const int SC_MANAGER_ALL_ACCESS = (STANDARD_RIGHTS_REQUIRED |
      SC_MANAGER_CONNECT |
      SC_MANAGER_CREATE_SERVICE |
      SC_MANAGER_ENUMERATE_SERVICE |
      SC_MANAGER_LOCK |
      SC_MANAGER_QUERY_LOCK_STATUS |
      SC_MANAGER_MODIFY_BOOT_CONFIG);

    private const int SERVICE_ALL_ACCESS = (STANDARD_RIGHTS_REQUIRED |
      SERVICE_QUERY_CONFIG |
      SERVICE_CHANGE_CONFIG |
      SERVICE_QUERY_STATUS |
      SERVICE_ENUMERATE_DEPENDENTS |
      SERVICE_START |
      SERVICE_STOP |
      SERVICE_PAUSE_CONTINUE |
      SERVICE_INTERROGATE |
      SERVICE_USER_DEFINED_CONTROL);

    private const int SERVICE_AUTO_START = 0x00000002;

    #endregion

    public static void InstallService(string svcName, string svcDispName, string svcUserName, string svcPassword)
    {

      StringBuilder sTemp = new StringBuilder(255);
      GetModuleFileName(0, sTemp, 255);
      string svcPath = sTemp.ToString();

      IntPtr sc_handle = IntPtr.Zero;

      try
      {

        sc_handle = OpenSCManager(null, null, SC_MANAGER_CREATE_SERVICE);

        if (sc_handle.ToInt32() != 0)
        {
          IntPtr sv_handle = CreateService(sc_handle, svcName, svcDispName, SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS, SERVICE_AUTO_START, SERVICE_ERROR_NORMAL, svcPath, null, 0, null, null, null);

          if (sv_handle.ToInt32() == 0)
          {
            throw new System.Exception("Could not install service");
          }

          SERVICE_DESCRIPTION description;

          description.lpDescription = "Alarmus Standard Alarm Source";

          int hResult = ChangeServiceDescription(sv_handle, SERVICE_CONFIG_DESCRIPTION, ref description);

        }
        else
        {
          throw new System.Exception("Could not open control manager");
        }

      }

      finally
      {

        if (sc_handle != IntPtr.Zero)
        {
          CloseServiceHandle(sc_handle);
        }

      }
    }

    public static void UnInstallService(string svcName)
    {

      IntPtr sc_hndl = OpenSCManager(null, null, SC_MANAGER_ALL_ACCESS);

      if (sc_hndl.ToInt32() == 0)
      {
        throw new System.Exception("Could not uninstall service");
      }

      IntPtr svc_hndl = OpenService(sc_hndl, svcName, SERVICE_DELETE);

      if (svc_hndl.ToInt32() == 0)
      {
        throw new System.Exception("Could not uninstall service");
      }

      int result = DeleteService(svc_hndl);

      if (result == 0)
      {
        throw new System.Exception("Could not uninstall service");
      }

      CloseServiceHandle(sc_hndl);

    }

    public static bool IsServiceInstalled(string svcName)
    {

      bool bIsInstalled = false;

      IntPtr sc_hndl = OpenSCManager(null, null, SC_MANAGER_ALL_ACCESS);

      if (sc_hndl.ToInt32() != 0)
      {
        IntPtr svc_hndl = OpenService(sc_hndl, svcName, SERVICE_QUERY_CONFIG);
        if (svc_hndl.ToInt32() != 0)
        {
          bIsInstalled = true;
          CloseServiceHandle(svc_hndl);
        }
       }

      CloseServiceHandle(sc_hndl);

      return bIsInstalled;

    }


  }



}
