using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Text;
using System.Globalization;
using System.Web.Script.Serialization;

namespace nsASASClient
{

  public class cTcpSocketClient
  {

    public int ConnectInterval;
    public int EchoTestInterval;
    public int AckEchoTimeout;

    public TcpClient tcpClient = null;

    private cTcpSocketClientThread ClientThread;

    public IPAddress ipPrimaryServerAddress = null;
    public IPAddress ipSecondaryServerAddresss = null;

    public int iPrimaryServerPort = 0;
    public int iSecondaryServerPort = 0;

    public bool ThreadIsRunning = false;

    public void Startup(string sPrimaryServerAddress, string sSecondaryServerAddresss, int iConnectInterval, int iEchoTestInterval, int iAckEchoTimeout)
    {

      ConnectInterval = iConnectInterval;
      EchoTestInterval = iEchoTestInterval;
      AckEchoTimeout = iAckEchoTimeout;

      try
      {

        if (TCPHelper.ResolveServerAddress(sPrimaryServerAddress, out ipPrimaryServerAddress, out iPrimaryServerPort) == false)
        {
          ASASClient.SysLog.SysLog("Invalid PrimaryServer address: {0}", sPrimaryServerAddress);
        }
        if (TCPHelper.ResolveServerAddress(sSecondaryServerAddresss, out ipSecondaryServerAddresss, out iSecondaryServerPort) == false)
        {
          ASASClient.SysLog.SysLog("Invalid SecondaryServer address: {0}", sSecondaryServerAddresss);
        }

        if (ipPrimaryServerAddress != null || ipSecondaryServerAddresss != null)
        {
          ThreadIsRunning = true;
          ClientThread = new cTcpSocketClientThread(this);
          new Thread(new ThreadStart(ClientThread.RunThread)).Start();
          ASASClient.SysLog.SysLog("Created client connection thread");
        }
      }

      catch (SocketException se)
      {
        ASASClient.SysLog.SysLog("Could not create client connection thread: " + se.Message);
      }

    }

    ~cTcpSocketClient()
    {
      Shutdown();
    }

    public void Shutdown()
    {

      lock (this)
      {
        TCPHelper.CloseAndDeleteStreamAndSocket(ref ClientThread.networkStream, ref ClientThread.tcpClient);
      }

      while (ThreadIsRunning)
      {
        Thread.Sleep(10);
      }
    }

    public void Disconnect()
    {
      lock (this)
      {
        if (ClientThread != null)
        {
          if (ClientThread.tcpClient != null)
          {
            if (ClientThread.networkStream != null)
            {
              TCPHelper.CloseAndDeleteStreamAndSocket(ref ClientThread.networkStream, ref ClientThread.tcpClient);
              ASASClient.SysLog.SysLog("We disconnected from RSMP client");
            }
          }
        }
      }
    }

		public void CyclicCleanup(int iElapsedMillisecs)
		{

			lock (this)
			{
				if (ClientThread != null)
				{
					if (ClientThread.tcpClient != null)
					{
						if (ClientThread.networkStream != null)
						{
							if (EchoTestInterval > 0)
							{
								if (ClientThread.bWeExcpectAnEchoAck == false)
								{
									ClientThread.iEchoTestTimer += iElapsedMillisecs;
									if (ClientThread.iEchoTestTimer >= EchoTestInterval)
									{
										ClientThread.bWeExcpectAnEchoAck = true;
										ClientThread.iEchoTestAckTimeoutTimer = 0;
										SendPacket(ASASClient.JSonSerializer.Serialize(new ASAS_JSon_Simple_Messages.EchoTest()));
										ClientThread.iEchoTestTimer = 0;
									}
								}
								else
								{
									ClientThread.iEchoTestAckTimeoutTimer += iElapsedMillisecs;
									if (ClientThread.iEchoTestAckTimeoutTimer >= AckEchoTimeout)
									{
										ASASClient.SysLog.SysLog("We did not receive EchoAck within {0} msecs - closing connection", AckEchoTimeout);
										TCPHelper.CloseAndDeleteStreamAndSocket(ref ClientThread.networkStream, ref ClientThread.tcpClient);
									}
								}
							}
						}
					}
				}
			}

		}

    public bool SendPacket(string sPacketData)
    {

      bool bSuccess = false;

      lock (this)
      {
        if (ClientThread != null)
        {
          if (ClientThread.tcpClient != null)
          {
            if (ClientThread.networkStream != null)
            {
              bSuccess = ClientThread.SendPacket (sPacketData);
            }
          }
        }
      }
      return bSuccess;
    }

  }

  //
  // Connect and client thread
  //
  class cTcpSocketClientThread
  {

    public const int BUFLENGTH = 10240;

    public cTcpSocketClient SocketClient;

    public TcpClient tcpClient;
    public NetworkStream networkStream;

    string sClientIP = "";

    byte[] inBuffer = new byte[BUFLENGTH];
    int inBufferLength;

    string sJSon;
    int iPacketLength;
    int iReadBytes;

		public int iEchoTestTimer;
		public int iEchoTestAckTimeoutTimer;
		public bool bWeExcpectAnEchoAck;

    public Encoding encoding = Encoding.Default;
    //public 

    public cTcpSocketClientThread(cTcpSocketClient tssSocketClient)
    {
      //
      // Remember our parent object
      //
      SocketClient = tssSocketClient;
      encoding = Encoding.GetEncoding("Windows-1252");
    }

    public bool SendPacket(string sPacket)
    {

      bool bSuccess = false;

      lock (this)
      {
        if (networkStream != null)
        {
          try
          {
            byte[] SendBytes = Encoding.UTF8.GetBytes(sPacket);
            byte[] bFormFeed = new byte[] { (byte)0x0c };
            SendByteArray(SendBytes, 0, SendBytes.GetLength(0), true, false);
            SendByteArray(bFormFeed, 0, 1, false, false);
            bSuccess = true;
          }
          catch (Exception e)
          {
            ASASClient.SysLog.SysLog("Could not send data: " + e.Message);
            TCPHelper.CloseAndDeleteStreamAndSocket(ref networkStream, ref tcpClient);
          }
        }
      }

      return bSuccess;

    }

    public bool SendByteArray(byte[] SendBuffer, int iOffset, int iLength, bool bNewPacket, bool bForceHexCode)
    {

      try
      {
        ASASClient.SysLog.Debug (SysLogAndDebug.Direction_Client2ASASServer, sClientIP, SendBuffer, iOffset, iLength, "", false);
        networkStream.Write(SendBuffer, iOffset, iLength);
        return true;
      }
      catch (SocketException e)
      {
        ASASClient.SysLog.SysLog("Could not send data: " + e.Message);
        TCPHelper.CloseAndDeleteStreamAndSocket(ref networkStream, ref tcpClient);
        return false;
      }
      catch (Exception e)
      {
        ASASClient.SysLog.SysLog("Could not send data: " + e.Message);
        TCPHelper.CloseAndDeleteStreamAndSocket(ref networkStream, ref tcpClient);
        return false;
      }
    }

    public void RunThread()
    {

      bool bTryUsingSecondaryNextTime = false;

      IPAddress ipNextIpAddress = (SocketClient.ipPrimaryServerAddress == null) ? SocketClient.ipSecondaryServerAddresss : SocketClient.ipPrimaryServerAddress;

      while (ASASClient.ThreadsShouldRun)
      {
        if (bTryUsingSecondaryNextTime)
        {
          if (SocketClient.ipSecondaryServerAddresss == null)
          {
            ConnectToServerAndMaintainStream (SocketClient.ipPrimaryServerAddress, SocketClient.iPrimaryServerPort);
          }
          else
          {
            ConnectToServerAndMaintainStream (SocketClient.ipSecondaryServerAddresss, SocketClient.iSecondaryServerPort );
          }
          bTryUsingSecondaryNextTime = false;
        }
        else
        {
          if (SocketClient.ipPrimaryServerAddress == null)
          {
            ConnectToServerAndMaintainStream (SocketClient.ipSecondaryServerAddresss, SocketClient.iSecondaryServerPort );
          }
          else
          {
            ConnectToServerAndMaintainStream (SocketClient.ipPrimaryServerAddress, SocketClient.iPrimaryServerPort);
          }
          bTryUsingSecondaryNextTime = true;
        }

        // Wait before we try to reconnect
        for (int iDelay = 0; iDelay < SocketClient.ConnectInterval && ASASClient.ThreadsShouldRun == true; iDelay += 100)
        {
          Thread.Sleep(100);
        }

      }

      lock (SocketClient)
      {
        SocketClient.ThreadIsRunning = false;
      }

    }

    private void ConnectToServerAndMaintainStream(IPAddress ipAddress, int iPort)
    {

      bool bFoundFormFeed;

      sClientIP = ipAddress.ToString() + ":" + iPort.ToString();

      try
      {
        tcpClient = new TcpClient();
        try
        {
          tcpClient.Connect(ipAddress, iPort);
        }
        catch (Exception e)
        {
          ASASClient.SysLog.SysLog("Could not connect ({0}): {1}", sClientIP, e.Message);
          TCPHelper.CloseAndDeleteStreamAndSocket(ref networkStream, ref tcpClient);
          return;
        }
        inBufferLength = 0;
				bWeExcpectAnEchoAck = false;
				iEchoTestTimer = 0;

        networkStream = tcpClient.GetStream();

        if (networkStream != null)
        {

          ASASClient.SysLog.SysLog("We connected to Alarmus Standard Alarm Source: {0}", sClientIP);

          while (ASASClient.ThreadsShouldRun)
          {
            // Read the server message into a byte buffer
            iReadBytes = networkStream.Read(inBuffer, inBufferLength, BUFLENGTH - inBufferLength);
            if (iReadBytes <= 0)
            {
              ASASClient.SysLog.SysLog("Alarmus Standard Alarm Source disconnected ({0})", sClientIP);
              break;
            }

            ASASClient.SysLog.Debug (SysLogAndDebug.Direction_ASASServer2Client, sClientIP, inBuffer, inBufferLength, iReadBytes, "", false);

            //TimeoutTimer = 0;
            inBufferLength += iReadBytes;

            do
            {
              bFoundFormFeed = false;
              for (iPacketLength = 0; iPacketLength < inBufferLength; iPacketLength++)
              {
                if (inBuffer[iPacketLength].CompareTo(0x0c) == 0)
                {
                  sJSon = Encoding.UTF8.GetString(inBuffer, 0, iPacketLength);
									ASAS_JSon_Simple_Messages.Header header;
									try
									{
										header = ASASClient.JSonSerializer.Deserialize<ASAS_JSon_Simple_Messages.Header>(sJSon);
										if (header.MsgType == null)
										{
											ASASClient.SysLog.SysLog("JSon header was bad: {0}", sJSon);
										}
										else
										{
											switch (header.MsgType.ToLower())
											{
												case "echotest":
													SendPacket(ASASClient.JSonSerializer.Serialize(new ASAS_JSon_Simple_Messages.EchoAck()));
													break;
												case "echoack":
													// If we don't get this in a while the ASAS server is probably dead
													bWeExcpectAnEchoAck = false;
													break;
												default:
													ASASClient.ReceivedPacketFromASASServer(sJSon);
													break;
											}
										}
									}
									catch
									{
										ASASClient.SysLog.SysLog("JSon header was bad: {0}", sJSon);
									}
                  iPacketLength++;
                  inBufferLength -= iPacketLength;
                  if (inBufferLength > 0)
                  {
                    Buffer.BlockCopy(inBuffer, iPacketLength, inBuffer, 0, inBufferLength);
                  }
                  bFoundFormFeed = true;
                  break;
                }
              }
            } while (bFoundFormFeed == true && inBufferLength > 0);

          }
        }
      }
      catch
      {
        if (networkStream != null)
        {
          ASASClient.SysLog.SysLog("Closed socket: {0}", sClientIP);
        }
      }
      TCPHelper.CloseAndDeleteStreamAndSocket(ref networkStream, ref tcpClient);

    }


  }


}